@@ -3,8 +3,8 @@
 "Sales extension for managing leads and opportunities"
 import datetime
 import time
-from trytond.model import ModelView, ModelSQL, ModelWorkflow, fields
-from trytond.wizard import Wizard
+from trytond.model import ModelView, ModelSQL, Workflow, fields
+from trytond.wizard import Wizard, StateView, StateAction, Button
 from trytond.backend import FIELDS
 from trytond.pyson import Equal, Eval, Not, In, If, Get, PYSONEncoder
 from trytond.transaction import Transaction
@@ -27,21 +27,21 @@ _STATES_STOP = {
 _DEPENDS_STOP = ['state']
 
 
-class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
+class SaleOpportunity(Workflow, ModelSQL, ModelView):
     'Sale Opportunity'
     _name = "sale.opportunity"
     _description = __doc__
     _history = True
     _rec_name = 'description'
 
-    party = fields.Many2One('party.party', 'Party', required=True, select=1,
+    party = fields.Many2One('party.party', 'Party', required=True, select=True,
         on_change=['party'], states=_STATES_STOP, depends=_DEPENDS_STOP)
     address = fields.Many2One('party.address', 'Address',
         domain=[('party', '=', Eval('party'))],
-        select=2, depends=['party', 'state'],
+        select=True, depends=['party', 'state'],
         states=_STATES_STOP)
     company = fields.Many2One('company.company', 'Company', required=True,
-        select=1, states=_STATES_STOP, domain=[
+        select=True, states=_STATES_STOP, domain=[
             ('id', If(In('company', Eval('context', {})), '=', '!='),
                 Get(Eval('context', {}), 'company', 0)),
             ], on_change=['company'], depends=_DEPENDS_STOP)
@@ -51,13 +51,6 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
             'get_currency_digits')
     amount = fields.Numeric('Amount', digits=(16, Eval('currency_digits', 2)),
             depends=['currency_digits'], help='Estimated revenue amount')
-    warehouse = fields.Many2One('stock.location', 'Warehouse',
-        domain=[('type', '=', 'warehouse')], states={
-            'required': Eval('state') == 'converted',
-            'readonly': In(Eval('state'),
-                ['converted', 'lost', 'cancelled']),
-            },
-        depends=['state'])
     payment_term = fields.Many2One('account.invoice.payment_term',
         'Payment Term', states={
             'required': Eval('state') == 'converted',
@@ -68,9 +61,9 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
     employee = fields.Many2One('company.employee', 'Employee', required=True,
             states=_STATES_STOP, depends=['state', 'company'],
             domain=[('company', '=', Eval('company'))])
-    start_date = fields.Date('Start Date', required=True, select=1,
+    start_date = fields.Date('Start Date', required=True, select=True,
         states=_STATES_START, depends=_DEPENDS_START)
-    end_date = fields.Date('End Date', select=2, readonly=True, states={
+    end_date = fields.Date('End Date', select=True, readonly=True, states={
         'invisible': Not(In(Eval('state'),
             ['converted', 'cancelled', 'lost'])),
     }, depends=['state'])
@@ -80,9 +73,9 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
         depends=_DEPENDS_STOP)
     lines = fields.One2Many('sale.opportunity.line', 'opportunity', 'Lines',
         states=_STATES_STOP, depends=_DEPENDS_STOP)
-    state = fields.Selection(STATES, 'State', required=True, select=1,
+    state = fields.Selection(STATES, 'State', required=True, select=True,
             sort=False, readonly=True)
-    probability = fields.Integer('Conversion Probability',
+    probability = fields.Integer('Conversion Probability', required=True,
             states={
                 'readonly': Not(In(Eval('state'), ['opportunity', 'lead'])),
             }, depends=['state'], help="Percentage between 0 and 100")
@@ -103,9 +96,41 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
                 'CHECK(probability >= 0 AND probability <= 100)',
                 'Probability must be between 0 and 100!')
         ]
-        self._rpc.update({
-            'button_lead': True,
-        })
+        self._error_messages.update({
+                'delete_cancel': 'Sale Opportunity "%s" must be cancelled '\
+                    'before deletion!',
+                })
+        self._transitions |= set((
+                ('lead', 'opportunity'),
+                ('lead', 'lost'),
+                ('lead', 'cancelled'),
+                ('opportunity', 'converted'),
+                ('opportunity', 'lead'),
+                ('opportunity', 'lost'),
+                ('opportunity', 'cancelled'),
+                ('lost', 'lead'),
+                ('cancelled', 'lead'),
+                ))
+        self._buttons.update({
+                'lead': {
+                    'invisible': ~Eval('state').in_(
+                        ['cancelled', 'lost', 'opportunity']),
+                    'icon': If(Eval('state').in_(['cancelled', 'lost']),
+                        'tryton-clear', 'tryton-go-previous'),
+                    },
+                'opportunity': {
+                    'invisible': ~Eval('state').in_(['lead']),
+                    },
+                'convert': {
+                    'invisible': ~Eval('state').in_(['opportunity']),
+                    },
+                'lost': {
+                    'invisible': ~Eval('state').in_(['lead', 'opportunity']),
+                    },
+                'cancel': {
+                    'invisible': ~Eval('state').in_(['lead', 'opportunity']),
+                    },
+                })
 
     def default_state(self):
         return 'lead'
@@ -118,34 +143,23 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
         return 50
 
     def default_company(self):
-        return Transaction().context.get('company') or False
+        return Transaction().context.get('company')
 
     def default_employee(self):
         user_obj = Pool().get('res.user')
-        employee_obj = Pool().get('company.employee')
 
-        employee_id = False
         if Transaction().context.get('employee'):
-            employee_id = Transaction().context['employee']
+            return Transaction().context['employee']
         else:
             user = user_obj.browse(Transaction().user)
             if user.employee:
-                employee_id = user.employee.id
-        return employee_id
+                return user.employee.id
 
     def default_payment_term(self):
         payment_term_obj = Pool().get('account.invoice.payment_term')
         payment_term_ids = payment_term_obj.search(self.payment_term.domain)
         if len(payment_term_ids) == 1:
             return payment_term_ids[0]
-        return False
-
-    def default_warehouse(self):
-        location_obj = Pool().get('stock.location')
-        location_ids = location_obj.search(self.warehouse.domain)
-        if len(location_ids) == 1:
-            return location_ids[0]
-        return False
 
     def get_currency(self, ids, name):
         res = {}
@@ -175,12 +189,12 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
         payment_term_obj = Pool().get('account.invoice.payment_term')
 
         res = {
-            'payment_term': False,
+            'payment_term': None,
         }
         if values.get('party'):
             party = party_obj.browse(values['party'])
-            if party.payment_term:
-                res['payment_term'] = party.payment_term.id
+            if party.customer_payment_term:
+                res['payment_term'] = party.customer_payment_term.id
         if not res['payment_term']:
             res['payment_term'] = self.default_payment_term()
         if res['payment_term']:
@@ -188,10 +202,6 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
                     res['payment_term']).rec_name
         return res
 
-    def button_lead(self, ids):
-        self.workflow_trigger_create(ids)
-        return True
-
     def _get_sale_line_opportunity_line(self, opportunity):
         '''
         Return sale line values for each opportunity line
@@ -204,6 +214,8 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
         line_obj = Pool().get('sale.opportunity.line')
         res = {}
         for line in opportunity.lines:
+            if line.sale_line:
+                continue
             val = line_obj.get_sale_line(line)
             if val:
                 res[line.id] = val
@@ -222,7 +234,6 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
             'party': opportunity.party.id,
             'payment_term': opportunity.payment_term.id,
             'company': opportunity.company.id,
-            'warehouse': opportunity.warehouse.id,
             'invoice_address': opportunity.address and opportunity.address.id,
             'shipment_address': opportunity.address and opportunity.address.id,
             'currency': opportunity.company.currency.id,
@@ -230,18 +241,17 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
         }
         return res
 
-    def create_sale(self, opportunity_id):
+    def create_sale(self, opportunity):
         '''
-        Create a sale for the opportunity
-
-        :param opportunity_id: the id of the opportunity
+        Create a sale for the opportunity and return the sale id
         '''
         pool = Pool()
         sale_obj = pool.get('sale.sale')
         sale_line_obj = pool.get('sale.line')
         line_obj = pool.get('sale.opportunity.line')
 
-        opportunity = self.browse(opportunity_id)
+        if opportunity.sale:
+            return
 
         values = self._get_sale_opportunity(opportunity)
         sale_line_values = self._get_sale_line_opportunity_line(opportunity)
@@ -257,37 +267,55 @@ class SaleOpportunity(ModelWorkflow, ModelSQL, ModelView):
                 'sale_line': sale_line_id,
                 })
 
-        self.write(opportunity_id, {
+        self.write(opportunity.id, {
             'sale': sale_id,
             })
         return sale_id
 
-    def wkf_lead(self, opportunity):
-        self.write(opportunity.id, {'state': 'lead'})
+    def delete(self, ids):
+        if isinstance(ids, (int, long)):
+            ids = [ids]
+        # Cancel before delete
+        self.cancel(ids)
+        for opportunity in self.browse(ids):
+            if opportunity.state != 'cancel':
+                self.raise_user_error('delete_cancel', opportunity.rec_name)
+        return super(SaleOpportunity, self).delete(ids)
 
-    def wkf_opportunity(self, opportunity):
-        self.write(opportunity.id, {'state': 'opportunity'})
+    @ModelView.button
+    @Workflow.transition('lead')
+    def lead(self, ids):
+        pass
 
-    def wkf_converted(self, opportunity):
+    @ModelView.button
+    @Workflow.transition('opportunity')
+    def opportunity(self, ids):
+        pass
+
+    @ModelView.button
+    @Workflow.transition('converted')
+    def convert(self, ids):
         date_obj = Pool().get('ir.date')
-        self.write(opportunity.id, {
+        self.write(ids, {
                 'end_date': date_obj.today(),
-                'state': 'converted',
                 })
-        self.create_sale(opportunity.id)
+        for opportunity in self.browse(ids):
+            self.create_sale(opportunity)
 
-    def wkf_cancelled(self, opportunity):
+    @ModelView.button
+    @Workflow.transition('lost')
+    def lost(self, ids):
         date_obj = Pool().get('ir.date')
-        self.write(opportunity.id, {
+        self.write(ids, {
                 'end_date': date_obj.today(),
-                'state': 'cancelled',
                 })
 
-    def wkf_lost(self, opportunity):
+    @ModelView.button
+    @Workflow.transition('cancelled')
+    def cancel(self, ids):
         date_obj = Pool().get('ir.date')
-        self.write(opportunity.id, {
+        self.write(ids, {
                 'end_date': date_obj.today(),
-                'state': 'lost',
                 })
 
 SaleOpportunity()
@@ -301,7 +329,7 @@ class SaleOpportunityLine(ModelSQL, ModelView):
     _history = True
 
     opportunity = fields.Many2One('sale.opportunity', 'Opportunity')
-    sequence = fields.Integer('Sequence')
+    sequence = fields.Integer('Sequence', required=True)
     product = fields.Many2One('product.product', 'Product', required=True,
             domain=[('salable', '=', True)], on_change=['product', 'unit'])
     quantity = fields.Float('Quantity', required=True,
@@ -311,7 +339,7 @@ class SaleOpportunityLine(ModelSQL, ModelView):
         on_change_with=['unit']), 'get_unit_digits')
     sale_line = fields.Many2One('sale.line', 'Sale Line', readonly=True,
         states={
-            'invisible': (Eval('_parent_opportunity', {}).get( 'state')
+            'invisible': (Eval('_parent_opportunity', {}).get('state')
                 != 'converted'),
             })
 
@@ -407,15 +435,14 @@ class SaleOpportunityHistory(ModelSQL, ModelView):
     }, depends=['state'])
     description = fields.Char('Description')
     comment = fields.Text('Comment')
-    lines = fields.Function(fields.One2Many('sale.opportunity.line', None, 'Lines',
-            datetime_field='date'), 'get_lines')
+    lines = fields.Function(fields.One2Many('sale.opportunity.line', None,
+            'Lines', datetime_field='date'), 'get_lines')
     state = fields.Selection(STATES, 'State')
     probability = fields.Integer('Conversion Probability')
     lost_reason = fields.Text('Reason for loss', states={
         'invisible': Not(Equal(Eval('state'), 'lost')),
     }, depends=['state'])
 
-
     def __init__(self):
         super(SaleOpportunityHistory, self).__init__()
         self._order.insert(0, ('date', 'DESC'))
@@ -426,9 +453,12 @@ class SaleOpportunityHistory(ModelSQL, ModelView):
         return [
             'MIN("%s".__id) AS id' % table,
             '"%s".id AS opportunity' % table,
-            'MIN(COALESCE("%s".write_date, "%s".create_date)) AS date' % (table, table),
-            'COALESCE("%s".write_uid, "%s".create_uid) AS user' % (table, table),
-        ] + ['"%s"."%s"' % (table, name) for name, field in self._columns.iteritems()
+            ('MIN(COALESCE("%s".write_date, "%s".create_date)) AS date'
+                % (table, table)),
+            ('COALESCE("%s".write_uid, "%s".create_uid) AS user'
+                % (table, table)),
+            ] + ['"%s"."%s"' % (table, name)
+                for name, field in self._columns.iteritems()
                 if name not in ('id', 'opportunity', 'date', 'user')
                 and not hasattr(field, 'set')]
 
@@ -463,7 +493,6 @@ class SaleOpportunityHistory(ModelSQL, ModelView):
             ])
         return result
 
-
     def read(self, ids, fields_names=None):
         res = super(SaleOpportunityHistory, self).read(ids,
                 fields_names=fields_names)
@@ -548,7 +577,8 @@ class SaleOpportunityEmployee(ModelSQL, ModelView):
         res = {}
         for record in self.browse(ids):
             if record.number:
-                res[record.id] = float(record.converted) / record.number * 100.0
+                res[record.id] = (float(record.converted)
+                    / record.number * 100.0)
             else:
                 res[record.id] = 0.0
         return res
@@ -578,50 +608,33 @@ class SaleOpportunityEmployee(ModelSQL, ModelView):
 SaleOpportunityEmployee()
 
 
-class OpenSaleOpportunityEmployeeInit(ModelView):
-    'Open Sale Opportunity per Employee Init'
-    _name = 'sale.open_opportunity_employee.init'
+class OpenSaleOpportunityEmployeeStart(ModelView):
+    'Open Sale Opportunity per Employee'
+    _name = 'sale.opportunity_employee.open.start'
     _description = __doc__
     start_date = fields.Date('Start Date')
     end_date = fields.Date('End Date')
 
-OpenSaleOpportunityEmployeeInit()
+OpenSaleOpportunityEmployeeStart()
 
 
 class OpenSaleOpportunityEmployee(Wizard):
     'Open Sale Opportunity per Employee'
-    _name = 'sale.open_opportunity_employee'
-    states = {
-        'init': {
-            'result': {
-                'type': 'form',
-                'object': 'sale.open_opportunity_employee.init',
-                'state': [
-                    ('end', 'Cancel', 'tryton-cancel'),
-                    ('open', 'Open', 'tryton-ok', True),
-                ],
-            },
-        },
-        'open': {
-            'result': {
-                'type': 'action',
-                'action': '_action_open',
-                'state': 'end',
-            },
-        },
-    }
+    _name = 'sale.opportunity_employee.open'
 
-    def _action_open(self, data):
-        model_data_obj = Pool().get('ir.model.data')
-        act_window_obj = Pool().get('ir.action.act_window')
-        act_window_id = model_data_obj.get_id('sale_opportunity',
-                'act_opportunity_employee_form')
-        res = act_window_obj.read(act_window_id)
-        res['pyson_context'] = PYSONEncoder().encode({
-            'start_date': data['form']['start_date'],
-            'end_date': data['form']['end_date'],
-            })
-        return res
+    start = StateView('sale.opportunity_employee.open.start',
+        'sale_opportunity.opportunity_employee_open_start_view_form', [
+            Button('Cancel', 'end', 'tryton-cancel'),
+            Button('Open', 'open_', 'tryton-ok', default=True),
+            ])
+    open_ = StateAction('sale_opportunity.act_opportunity_employee_form')
+
+    def do_open_(self, session, action):
+        action['pyson_context'] = PYSONEncoder().encode({
+                'start_date': session.start.start_date,
+                'end_date': session.start.end_date,
+                })
+        return action, {}
 
 OpenSaleOpportunityEmployee()
 
@@ -701,7 +714,8 @@ class SaleOpportunityMonthly(ModelSQL, ModelView):
         res = {}
         for record in self.browse(ids):
             if record.number:
-                res[record.id] = float(record.converted) / record.number * 100.0
+                res[record.id] = (float(record.converted)
+                    / record.number * 100.0)
             else:
                 res[record.id] = 0.0
         return res
@@ -815,7 +829,8 @@ class SaleOpportunityEmployeeMonthly(ModelSQL, ModelView):
         res = {}
         for record in self.browse(ids):
             if record.number:
-                res[record.id] = float(record.converted) / record.number * 100.0
+                res[record.id] = (float(record.converted)
+                    / record.number * 100.0)
             else:
                 res[record.id] = 0.0
         return res
